#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ObjectReference import ObjectReference
from kubernetes.models.v1.AWSElasticBlockStoreVolumeSource import AWSElasticBlockStoreVolumeSource
from kubernetes.models.v1.EmptyDirVolumeSource import EmptyDirVolumeSource
from kubernetes.models.v1.GCEPersistentDiskVolumeSource import GCEPersistentDiskVolumeSource
from kubernetes.models.v1.GitRepoVolumeSource import GitRepoVolumeSource
from kubernetes.models.v1.HostPathVolumeSource import HostPathVolumeSource
from kubernetes.models.v1.NFSVolumeSource import NFSVolumeSource
from kubernetes.models.v1.SecretVolumeSource import SecretVolumeSource
from kubernetes.models.v1.PersistentVolumeClaimVolumeSource import PersistentVolumeClaimVolumeSource
from kubernetes.utils import is_valid_string, is_valid_dict, is_valid_list


class PersistentVolumeSpec(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_persistentvolumespec
    """

    VALID_CAPACITY_PARAMS = ['storage']
    VALID_ACCESS_MODES = ['ReadWriteOnce', 'ReadOnlyMany', 'ReadWriteMany']
    VALID_RECLAIM_POLICIES = ['Retain', 'Recycle', 'Delete']

    VOLUME_TYPES_TO_SOURCE_MAP = {
        'awsElasticBlockStore': AWSElasticBlockStoreVolumeSource,
        'emptyDir': EmptyDirVolumeSource,
        'gcePersistentDisk': GCEPersistentDiskVolumeSource,
        'gitRepo': GitRepoVolumeSource,
        'hostPath': HostPathVolumeSource,
        'nfs': NFSVolumeSource,
        'secret': SecretVolumeSource
    }

    def __init__(self, model=None):
        super(PersistentVolumeSpec, self).__init__()

        # TODO(froch): add support for the below
        # self._iscsi = None
        # self._glusterfs = None
        # self._rbd = None
        # self._flex_volume = None
        # self._cinder = None
        # self._cephfs = None
        # self._flocker = None
        # self._downward_api = None
        # self._fc = None
        # self._azure_file = None
        # self._config_map = None
        # self._vsphere_volume
        # self._quobyte = None
        # self._azuredisk = None

        self._awsElasticBlockStore = None
        self._emptyDir = None
        self._gcePersistentDisk = None
        self._gitRepo = None
        self._hostPath = None
        self._name = None
        self._nfs = None
        self._persistentVolumeClaim = None
        self._secret = None

        self._capacity = {'storage': '10Gi'}
        self._access_modes = ['ReadWriteOnce']
        self._claim_ref = None
        self._reclaim_policy = 'Retain'
        self._storage_class_name = ""

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'awsElasticBlockStore' in model:
            self.awsElasticBlockStore = AWSElasticBlockStoreVolumeSource(model['awsElasticBlockStore'])
        if 'gcePersistentDisk' in model:
            self.gcePersistentDisk = GCEPersistentDiskVolumeSource(model['gcePersistentDisk'])
        if 'hostPath' in model:
            self.hostPath = HostPathVolumeSource(model['hostPath'])
        if 'name' in model:
            self.name = model['name']
        if 'nfs' in model:
            self.nfs = NFSVolumeSource(model['nfs'])
        if 'secret' in model:
            self.secret = SecretVolumeSource(model['secret'])
        if 'capacity' in model:
            self.capacity = model['capacity']
        if 'accessModes' in model:
            self.access_modes = model['accessModes']
        if 'claimRef' in model:
            self.claim_ref = ObjectReference(model['claimRef'])
        if 'persistentVolumeReclaimPolicy' in model:
            self.reclaim_policy = model['persistentVolumeReclaimPolicy']
        if 'persistentVolumeClaim' in model:
            self.persistentVolumeClaim = PersistentVolumeClaimVolumeSource(model['persistentVolumeClaim'])
        if 'storageClassName' in model:
            self._storage_class_name = model['storageClassName']

    # ------------------------------------------------------------------------------------- aws ebs

    @property
    def awsElasticBlockStore(self):
        return self._awsElasticBlockStore

    @awsElasticBlockStore.setter
    def awsElasticBlockStore(self, ebs=None):
        if not isinstance(ebs, AWSElasticBlockStoreVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: aws_elastic_block_store: [ {0} ] is invalid.'.format(ebs))
        self._awsElasticBlockStore = ebs

    # ------------------------------------------------------------------------------------- emptyDir

    @property
    def emptyDir(self):
        return self._emptyDir

    @emptyDir.setter
    def emptyDir(self, edir=None):
        if not isinstance(edir, EmptyDirVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: empty_dir: [ {0} ] is invalid.'.format(edir))
        self._emptyDir = edir

    # ------------------------------------------------------------------------------------- gce pd

    @property
    def gcePersistentDisk(self):
        return self._gcePersistentDisk

    @gcePersistentDisk.setter
    def gcePersistentDisk(self, pd=None):
        if not isinstance(pd, GCEPersistentDiskVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: gce_persistent_disk: [ {0} ] is invalid.'.format(pd))
        self._gcePersistentDisk = pd

    # ------------------------------------------------------------------------------------- gitRepo

    @property
    def gitRepo(self):
        return self._gitRepo

    @gitRepo.setter
    def gitRepo(self, repo=None):
        if not isinstance(repo, GitRepoVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: git_repo: [ {0} ] is invalid.'.format(repo))
        self._gitRepo = repo

    # ------------------------------------------------------------------------------------- hostPath

    @property
    def hostPath(self):
        return self._hostPath

    @hostPath.setter
    def hostPath(self, hp=None):
        if not isinstance(hp, HostPathVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: host_path: [ {0} ] is invalid.'.format(hp))
        self._hostPath = hp

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('PersistentVolumeSpec: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- nfs

    @property
    def nfs(self):
        return self._nfs

    @nfs.setter
    def nfs(self, nfs=None):
        if not isinstance(nfs, NFSVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: nfs: [ {0} ] is invalid.'.format(nfs))
        self._nfs = nfs

    # ------------------------------------------------------------------------------------- secret

    @property
    def secret(self):
        return self._secret

    @secret.setter
    def secret(self, secret=None):
        if not isinstance(secret, SecretVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: secret: [ {0} ] is invalid.'.format(secret))
        self._secret = secret

    # ------------------------------------------------------------------------------------- capacity

    @property
    def capacity(self):
        return self._capacity

    @capacity.setter
    def capacity(self, c=None):
        if not is_valid_dict(c, PersistentVolumeSpec.VALID_CAPACITY_PARAMS):
            raise SyntaxError('PersistentVolumeSpec: capacity: [ {} ] is invalid.'.format(c))
        self._capacity = c

    # ------------------------------------------------------------------------------------- accessModes

    @property
    def access_modes(self):
        return self._access_modes

    @access_modes.setter
    def access_modes(self, modes=None):
        if not is_valid_list(modes, str):
            raise SyntaxError('PersistentVolumeSpec: access_modes: [ {} ] is invalid.'.format(modes))
        filtered = list(filter(lambda x: x in PersistentVolumeSpec.VALID_ACCESS_MODES, modes))
        self._access_modes = filtered

    # ------------------------------------------------------------------------------------- claimRef

    @property
    def claim_ref(self):
        return self._claim_ref

    @claim_ref.setter
    def claim_ref(self, ref=None):
        if not isinstance(ref, ObjectReference):
            raise SyntaxError('PersistentVolumeSpec: claim_ref: [ {} ] is invalid.'.format(ref))
        self._claim_ref = ref

    # ------------------------------------------------------------------------------------- reclaimPolicy

    @property
    def reclaim_policy(self):
        return self._reclaim_policy

    @reclaim_policy.setter
    def reclaim_policy(self, policy=None):
        if policy not in PersistentVolumeSpec.VALID_RECLAIM_POLICIES:
            raise SyntaxError('PersistentVolumeSpec: reclaim_policy: [ {} ] is invalid.'.format(policy))
        self._reclaim_policy = policy

    # ------------------------------------------------------------------------------------- persistentVolumeClaim

    @property
    def persistentVolumeClaim(self):
        return self._persistentVolumeClaim

    @persistentVolumeClaim.setter
    def persistentVolumeClaim(self, pvc=None):
        if not isinstance(pvc, PersistentVolumeClaimVolumeSource):
            raise SyntaxError('PersistentVolumeSpec: persistentVolumeClaim: [ {} ] is invalid.'.format(pvc))
        self._persistentVolumeClaim = pvc

    # ------------------------------------------------------------------------------------- storageClassName

    @property
    def storage_class_name(self):
        return self._storage_class_name

    @storage_class_name.setter
    def storage_class_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('PersistentVolumeSpec: storage_class_name: [ {} ] is invalid.'.format(name))
        self._storage_class_name = name

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.awsElasticBlockStore is not None:
            data['awsElasticBlockStore'] = self.awsElasticBlockStore.serialize()
        if self.emptyDir is not None:
            data['emptyDir'] = self.emptyDir.serialize()
        if self.gcePersistentDisk is not None:
            data['gcePersistentDisk'] = self.gcePersistentDisk.serialize()
        if self.gitRepo is not None:
            data['gitRepo'] = self.gitRepo.serialize()
        if self.hostPath is not None:
            data['hostPath'] = self.hostPath.serialize()
        if self.name is not None:
            data['name'] = self.name
        if self.nfs is not None:
            data['nfs'] = self.nfs.serialize()
        if self.secret is not None:
            data['secret'] = self.secret.serialize()
        if self.capacity is not None:
            data['capacity'] = self.capacity
        if self.access_modes is not None:
            data['accessModes'] = self.access_modes
        if self.claim_ref is not None:
            data['claimRef'] = self.claim_ref.serialize()
        if self.reclaim_policy is not None:
            data['persistentVolumeReclaimPolicy'] = self.reclaim_policy
        if self.persistentVolumeClaim is not None:
            data['persistentVolumeClaim'] = self.reclaim_policy
        if self.storage_class_name is not None:
            data['storageClassName'] = self.storage_class_name
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.utils import is_valid_string, is_valid_dict


class StorageClass(BaseModel):
    """
    https://github.com/kubernetes/kubernetes/tree/master/examples/persistent-volume-provisioning/
    https://kubernetes.io/docs/user-guide/persistent-volumes/#storageclasses
    """

    def __init__(self, model=None):
        super(StorageClass, self).__init__()

        self._kind = 'StorageClass'
        self._api_version = 'storage.k8s.io/v1beta1'

        self._metadata = ObjectMeta()
        self._provisioner = None
        self._parameters = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(StorageClass, self).build_with_model(model)

        if 'provisioner' in model:
            self.provisioner = model['provisioner']
        if 'parameters' in model:
            self.parameters = model['parameters']

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('StorageClass: kind: [ {} ] is invalid.'.format(k))
        self._kind = k

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('StorageClass: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- metadata

    @property
    def metadata(self):
        return self._metadata

    @metadata.setter
    def metadata(self, meta=None):
        if not isinstance(meta, ObjectMeta):
            raise SyntaxError('StorageClass: metadata: [ {} ] is invalid.'.format(meta))
        self._metadata = meta

    # ------------------------------------------------------------------------------------- provisioner

    @property
    def provisioner(self):
        return self._provisioner

    @provisioner.setter
    def provisioner(self, p=None):
        if not is_valid_string(p):
            raise SyntaxError('StorageClass: provisioner: [ {} ] is invalid.'.format(p))
        self._provisioner = p

    # ------------------------------------------------------------------------------------- parameters

    @property
    def parameters(self):
        return self._parameters

    @parameters.setter
    def parameters(self, p=None):
        if not is_valid_dict(p):
            raise SyntaxError('StorageClass: parameters: [ {} ] is invalid.'.format(p))
        self._parameters = p

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(StorageClass, self).serialize()

        if self.provisioner is not None:
            data['provisioner'] = self.provisioner
        if self.parameters is not None:
            data['parameters'] = self.parameters
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model


class NodeSystemInfo(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodesysteminfo
    """

    def __init__(self, model=None):
        super(NodeSystemInfo, self).__init__()

        self._machine_id = None
        self._system_uuid = None
        self._boot_id = None
        self._kernel_version = None
        self._os_image = None
        self._container_runtime_version = None
        self._kubelet_version = None
        self._kube_proxy_version = None
        self._operating_system = None
        self._architecture = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'machineID' in model:
            self.machine_id = model['machineID']
        if 'systemUUID' in model:
            self.system_uuid = model['systemUUID']
        if 'bootID' in model:
            self.boot_id = model['bootID']
        if 'kernelVersion' in model:
            self.kernel_version = model['kernelVersion']
        if 'osImage' in model:
            self.os_image = model['osImage']
        if 'containerRuntimeVersion' in model:
            self.container_runtime_version = model['containerRuntimeVersion']
        if 'kubeletVersion' in model:
            self.kubelet_version = model['kubeletVersion']
        if 'kubeProxyVersion' in model:
            self.kube_proxy_version = model['kubeProxyVersion']
        if 'operatingSystem' in model:
            self.operating_system = model['operatingSystem']
        if 'architecture' in model:
            self.architecture = model['architecture']

    # ------------------------------------------------------------------------------------- machine_id

    @property
    def machine_id(self):
        return self._machine_id

    @machine_id.setter
    def machine_id(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: machine_id: [ {0} ] is invalid.'.format(v))
        self._machine_id = v

    # ------------------------------------------------------------------------------------- machine_id

    @property
    def system_uuid(self):
        return self._system_uuid

    @system_uuid.setter
    def system_uuid(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: system_uuid: [ {0} ] is invalid.'.format(v))
        self._system_uuid = v

    # ------------------------------------------------------------------------------------- boot_id

    @property
    def boot_id(self):
        return self._boot_id

    @boot_id.setter
    def boot_id(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: boot_id: [ {0} ] is invalid.'.format(v))
        self._boot_id = v

    # ------------------------------------------------------------------------------------- kernel_version

    @property
    def kernel_version(self):
        return self._kernel_version

    @kernel_version.setter
    def kernel_version(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: kernel_version: [ {0} ] is invalid.'.format(v))
        self._kernel_version = v

    # ------------------------------------------------------------------------------------- os_image

    @property
    def os_image(self):
        return self._os_image

    @os_image.setter
    def os_image(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: os_image: [ {0} ] is invalid.'.format(v))
        self._os_image = v

    # ------------------------------------------------------------------------------------- container_runtime_version

    @property
    def container_runtime_version(self):
        return self._container_runtime_version

    @container_runtime_version.setter
    def container_runtime_version(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: container_runtime_version: [ {0} ] is invalid.'.format(v))
        self._container_runtime_version = v

    # ------------------------------------------------------------------------------------- kubelet_version

    @property
    def kubelet_version(self):
        return self._kubelet_version

    @kubelet_version.setter
    def kubelet_version(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: kubelet_version: [ {0} ] is invalid.'.format(v))
        self._kubelet_version = v

    # ------------------------------------------------------------------------------------- kube_proxy_version

    @property
    def kube_proxy_version(self):
        return self._kube_proxy_version

    @kube_proxy_version.setter
    def kube_proxy_version(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: kube_proxy_version: [ {0} ] is invalid.'.format(v))
        self._kube_proxy_version = v

    # ------------------------------------------------------------------------------------- operating_system

    @property
    def operating_system(self):
        return self._operating_system

    @operating_system.setter
    def operating_system(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: operating_system: [ {0} ] is invalid.'.format(v))
        self._operating_system = v

    # ------------------------------------------------------------------------------------- architecture

    @property
    def architecture(self):
        return self._architecture

    @architecture.setter
    def architecture(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeSystemInfo: architecture: [ {0} ] is invalid.'.format(v))
        self._architecture = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.machine_id:
            data['machineID'] = self.machine_id
        if self.system_uuid:
            data['systemUUID'] = self.system_uuid
        if self.boot_id:
            data['bootID'] = self.boot_id
        if self.kernel_version:
            data['kernelVersion'] = self.kernel_version
        if self.os_image:
            data['osImage'] = self.os_image
        if self.container_runtime_version:
            data['containerRuntimeVersion'] = self.container_runtime_version
        if self.kubelet_version:
            data['kubeletVersion'] = self.kubelet_version
        if self.kube_proxy_version:
            data['kubeProxyVersion'] = self.kube_proxy_version
        if self.operating_system:
            data['operatingSystem'] = self.operating_system
        if self.architecture:
            data['architecture'] = self.architecture
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ContainerStatus import ContainerStatus
from kubernetes.models.v1.PodCondition import PodCondition
from kubernetes.utils import is_valid_list, is_valid_string, filter_model


class PodStatus(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_podstatus
    """

    def __init__(self, model=None):
        super(PodStatus, self).__init__()

        self._conditions = []
        self._container_statuses = []
        self._phase = None
        self._message = None
        self._reason = None
        self._host_ip = None
        self._pod_ip = None
        self._start_time = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'conditions' in model:
            conditions = []
            for c in model['conditions']:
                condition = PodCondition(c)
                conditions.append(condition)
            self.conditions = conditions
        if 'containerStatuses' in model:
            statuses = []
            for s in model['containerStatuses']:
                status = ContainerStatus(s)
                statuses.append(status)
            self.container_statuses = statuses
        if 'phase' in model:
            self.phase = model['phase']
        if 'message' in model:
            self.message = model['message']
        if 'reason' in model:
            self.reason = model['reason']
        if 'hostIP' in model:
            self.host_ip = model['hostIP']
        if 'podIP' in model:
            self.pod_ip = model['podIP']
        if 'startTime' in model:
            self.start_time = model['startTime']

    # ------------------------------------------------------------------------------------- add

    def add_container_status(self, status=None):
        if not isinstance(status, ContainerStatus):
            raise SyntaxError('PodStatus: status: [ {0} ] is invalid.'.format(status))
        self._container_statuses.append(status)

    # ------------------------------------------------------------------------------------- conditions

    @property
    def conditions(self):
        return self._conditions

    @conditions.setter
    def conditions(self, conditions=None):
        if not is_valid_list(conditions, PodCondition):
            raise SyntaxError('PodStatus: conditions: [ {0} ] is invalid.'.format(conditions))
        self._conditions = conditions

    # ------------------------------------------------------------------------------------- container status

    @property
    def container_statuses(self):
        return self._container_statuses

    @container_statuses.setter
    def container_statuses(self, statuses=None):
        if not is_valid_list(statuses, ContainerStatus):
            raise SyntaxError('PodStatus: container_statuses: [ {0} ] is invalid.'.format(statuses))
        self._container_statuses = statuses

    # ------------------------------------------------------------------------------------- phase

    @property
    def phase(self):
        return self._phase

    @phase.setter
    def phase(self, phase=None):
        if not is_valid_string(phase):
            raise SyntaxError('PodStatus: container_statuses: [ {0} ] is invalid.'.format(phase))
        self._phase = phase

    # ------------------------------------------------------------------------------------- phase

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, message=None):
        if not is_valid_string(message):
            raise SyntaxError('PodStatus: message: [ {0} ] is invalid.'.format(message))
        self._message = message

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, reason=None):
        if not is_valid_string(reason):
            raise SyntaxError('PodStatus: reason: [ {0} ] is invalid.'.format(reason))
        self._reason = reason

    # ------------------------------------------------------------------------------------- hostIP

    @property
    def host_ip(self):
        return self._host_ip

    @host_ip.setter
    def host_ip(self, ip=None):
        if not is_valid_string(ip):
            raise SyntaxError('PodStatus: host_ip: [ {0} ] is invalid.'.format(ip))
        self._host_ip = ip

    # ------------------------------------------------------------------------------------- podIP

    @property
    def pod_ip(self):
        return self._pod_ip

    @pod_ip.setter
    def pod_ip(self, ip=None):
        if not is_valid_string(ip):
            raise SyntaxError('PodStatus: pod_ip: [ {0} ] is invalid.'.format(ip))
        self._pod_ip = ip

    # ------------------------------------------------------------------------------------- start time

    @property
    def start_time(self):
        return self._start_time

    @start_time.setter
    def start_time(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('PodStatus: start_time: [ {0} ] is invalid.'.format(time))
        self._start_time = time

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.phase is not None:
            data['phase'] = self.phase
        if self.conditions:
            data['conditions'] = [x.serialize() for x in self.conditions]
        if self.message is not None:
            data['message'] = self.message
        if self.reason is not None:
            data['reason'] = self.reason
        if self.host_ip is not None:
            data['hostIP'] = self.host_ip
        if self.pod_ip is not None:
            data['podIP'] = self.pod_ip
        if self.start_time is not None:
            data['startTime'] = self.start_time
        if self.container_statuses:
            data['containerStatuses'] = [x.serialize() for x in self.container_statuses]
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json
import time

from kubernetes.K8sExceptions import DrainNodeException, TimedOutException
from kubernetes.K8sNamespace import K8sNamespace
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sPod import K8sPod
from kubernetes.models.v1.Node import Node
from kubernetes.utils import is_valid_string, is_valid_list
from kubernetes.models.v1.Taint import Taint


class K8sNode(K8sObject):

    DRAIN_WAIT_TIMEOUT_SECONDS = 60

    def __init__(self, config=None, name=None):
        super(K8sNode, self).__init__(
            config=config,
            name=name,
            obj_type='Node'
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sNode, self).create()
        self.get()
        return self

    def update(self):
        super(K8sNode, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sNode, self).list()
        nodes = list(map(lambda x: Node(x), ls))
        if pattern is not None:
            nodes = list(filter(lambda x: pattern in x.name, nodes))
        k8s = []
        for x in nodes:
            j = K8sNode(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = Node(self.get_model())
        return self

    def get_annotation(self, k=None):
        if k in self.model.metadata.annotations:
            return self.model.metadata.annotations[k]
        return None

    def get_label(self, k=None):
        if k in self.model.metadata.labels:
            return self.model.metadata.labels[k]
        return None

    # ------------------------------------------------------------------------------------- pod_cidr

    @property
    def pod_cidr(self):
        return self.model.spec.pod_cidr

    @pod_cidr.setter
    def pod_cidr(self, v=None):
        self.model.spec.pod_cidr = v

    # ------------------------------------------------------------------------------------- external_id

    @property
    def external_id(self):
        return self.model.spec.external_id

    @external_id.setter
    def external_id(self, v=None):
        self.model.spec.external_id = v

    # ------------------------------------------------------------------------------------- provider_id

    @property
    def provider_id(self):
        return self.model.spec.provider_id

    @provider_id.setter
    def provider_id(self, v=None):
        self.model.spec.provider_id = v

    # ------------------------------------------------------------------------------------- unschedulable

    @property
    def unschedulable(self):
        return self.model.spec.unschedulable

    @unschedulable.setter
    def unschedulable(self, v=None):
        self.model.spec.unschedulable = v

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self.model.status

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.metadata.name

    @name.setter
    def name(self, name=None):
        self.model.metadata.name = name

    # ------------------------------------------------------------------------------------- filter

    @staticmethod
    def get_by_name(config=None, name=None):
        nodes = K8sNode(config=config, name=name).list()
        filtered = list(filter(lambda x: x.name == name, nodes))
        if filtered:
            return filtered[0]
        return None

    # ------------------------------------------------------------------------------------- pods

    @property
    def pods(self):
        return self._pod_inventory()

    @pods.setter
    def pods(self, p=None):
        raise NotImplementedError("K8sNode: pods is read-only.")

    # ------------------------------------------------------------------------------------- drain

    def drain(self, ignore_daemonsets=False, delete_local_storage=False, force=False):
        """
        Removes all K8sPods from this K8sNode, 
        and prevents additional K8sPods from being scheduled.
        
        :param ignore_daemonsets: a boolean. 
                If false, will fail if a K8sDaemonSet-managed K8sPod is present.
                If true, will continue even if a K8sDaemonSet-managed K8sPod is present.
                        
        :param delete_local_storage: a boolean.
                If false, will fail if a K8sVolume of type 'emptyDir' is found.
                If true, will continue even if an 'emptyDir' K8sVolume is found.
        
        :param force: a boolean.
                If false, will fail if any K8sPods unmanaged by a parent object are found.
                If true, will continue and any unmanaged K8sPods are lost.
        
        :return: self. 
        """

        # inventory of K8sPods found on this node.
        daemonset_pods = []
        pods = self._pod_inventory()

        # cordon the node.
        self.unschedulable = True
        self.update()

        # loop through all pods and delete them.
        for pod in pods:
            if self._is_daemonset(pod):
                if not ignore_daemonsets:
                    raise DrainNodeException("K8sNode: pod: [ {} ] is managed by a DaemonSet.".format(pod.name))
                else:
                    daemonset_pods.append(pod)
                    continue
            if self._has_local_storage(pod) and not delete_local_storage:
                raise DrainNodeException("K8sNode: pod: [ {} ] has local storage that will be lost.".format(pod.name))
            if self._is_orphan(pod) and not force:
                raise DrainNodeException("K8sNode: pod: [ {} ] is unmanaged and will be lost.".format(pod.name))

            pod.delete()

        self._wait_for_pod_deletion(daemonset_pods)

        return self

    def _pod_inventory(self):
        """
        Returns the list of all K8sPods found on this K8sNode.
        
        :return: A list of K8sPods.
        """

        pods = []
        namespaces = K8sNamespace(config=self.config, name="yo").list()
        for ns in namespaces:
            cfg = self.config
            cfg.namespace = ns.name
            p = K8sPod(config=cfg, name="yo").list()
            filtered = filter(lambda x: x.node_name == self.name, p)
            pods += filtered
        return pods

    def _is_daemonset(self, pod=None):
        """
        Determines if a K8sPod is part of a K8sDaemonSet.
        
        :param pod: The K8sPod we're interested in.

        :return: a boolean.
        """

        if 'kubernetes.io/created-by' in pod.annotations:
            parent = json.loads(pod.annotations['kubernetes.io/created-by'])
            if parent['reference']['kind'] == 'DaemonSet':
                return True
        return False

    def _has_local_storage(self, pod=None):
        """
        Determines if a K8sPod has any local storage susceptible to be lost.

        :param pod: The K8sPod we're interested in.

        :return: a boolean.
        """

        for vol in pod.volumes:
            if vol.emptyDir is not None:
                return True
        return False

    def _is_orphan(self, pod=None):
        """
        Determines if a K8sPod is unmanaged by a parent object, and is susceptible to be lost.

        :param pod: The K8sPod we're interested in.

        :return: a boolean.
        """

        if 'kubernetes.io/created-by' not in pod.annotations:
            return True
        return False

    def _wait_for_pod_deletion(self, daemonset_pods=None):
        """
        Wait until this K8sNode has evicted all its K8sPods.
        
        :param daemonset_pods: A list of K8sPods on this K8sNode that are managed by a K8sDaemonSet.
        
        :return: None
        """

        pods = self._pod_inventory()
        start_time = time.time()
        while len(pods) > 0:
            if len(pods) == len(daemonset_pods):
                break
            pods = self._pod_inventory()
            self._check_timeout(start_time)
            time.sleep(1)
        return

    def _check_timeout(self, start_time=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= self.DRAIN_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException("Timed out draining K8sNode: [ {0} ]".format(self.name))

    # ------------------------------------------------------------------------------------- uncordon

    def uncordon(self):
        """
        Returns this K8sNode into the pool addressable by the kube-scheduler.
        
        :return: self
        """

        self.unschedulable = False
        self.update()
        return self

    # ------------------------------------------------------------------------------------- taint

    @property
    def taints(self):
        return self.model.spec.taints

    @taints.setter
    def taints(self, t=None):
        if not is_valid_list(t, Taint):
            raise SyntaxError('K8sNode: taints: [ {} ] is invalid.'.format(t))
        self.model.spec.taints = t

    def taint(self, key=None, value=None, effect=None):
        if not (key and value and effect):
            raise SyntaxError('K8sNode: taint: you must specify a key, a value and an effect.')
        if not is_valid_string(key) or not is_valid_string(value):
            raise SyntaxError('K8sNode: taint: key: [ {} ] or value: [ {} ] is invalid.'.format(key, value))
        if effect not in Taint.VALID_TAINT_EFFECTS:
            raise SyntaxError('K8sNode: taint: effect must be in {}'.format(Taint.VALID_TAINT_EFFECTS))

        t = Taint()
        t.key = key
        t.value = value
        t.effect = effect

        exists = False
        for existing_taint in self.taints:
            if existing_taint.key == key and existing_taint.value == value and existing_taint.effect == effect:
                exists = True
        if not exists:
            self.taints.append(t)
            self.update()
        return self

    def untaint(self, key=None, value=None):
        if key and value:
            if not is_valid_string(key) or not is_valid_string(value):
                raise SyntaxError('K8sNode: taint: key: [ {} ] or value: [ {} ] is invalid.'.format(key, value))

        remaining_taints = []
        for t in self.taints:
            if key and value:
                if t.key != key and t.value != value:
                    remaining_taints.append(t)

        self.taints = remaining_taints
        self.update()
        return self



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.Capabilities import Capabilities
from kubernetes.models.v1.SELinuxOptions import SELinuxOptions
from kubernetes.utils import filter_model


class SecurityContext(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_securitycontext
    """

    def __init__(self, model=None):
        super(SecurityContext, self).__init__()

        self._capabilities = None
        self._privileged = None
        self._read_only_root_filesystem = None
        self._run_as_non_root = None
        self._run_as_user = None
        self._se_linux_options = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'capabilities' in model:
            self.capabilities = Capabilities(model['capabilities'])
        if 'privileged' in model:
            self.privileged = model['privileged']
        if 'readOnlyRootFilesystem' in model:
            self.read_only_root_filesystem = model['readOnlyRootFilesystem']
        if 'runAsNonRoot' in model:
            self.run_as_non_root = model['runAsNonRoot']
        if 'runAsUser' in model:
            self.run_as_user = model['runAsUser']

    # ------------------------------------------------------------------------------------- capabilities

    @property
    def capabilities(self):
        return self._capabilities

    @capabilities.setter
    def capabilities(self, c=None):
        if not isinstance(c, Capabilities):
            raise SyntaxError('SecurityContext: capabilities: [ {} ] is invalid.'.format(c))
        self._capabilities = c

    # ------------------------------------------------------------------------------------- privileged

    @property
    def privileged(self):
        return self._privileged

    @privileged.setter
    def privileged(self, p=None):
        if not isinstance(p, bool):
            raise SyntaxError('SecurityContext: privileged: [ {} ] is invalid.'.format(p))
        self._privileged = p

    # ------------------------------------------------------------------------------------- readOnlyRootFilesystem

    @property
    def read_only_root_filesystem(self):
        return self._read_only_root_filesystem

    @read_only_root_filesystem.setter
    def read_only_root_filesystem(self, r=None):
        if not isinstance(r, bool):
            raise SyntaxError('SecurityContext: read_only_root_filesystem: [ {} ] is invalid.'.format(r))
        self._read_only_root_filesystem = r

    # ------------------------------------------------------------------------------------- runAsNonRoot

    # May also be set in PodSecurityContext.
    # If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.

    @property
    def run_as_non_root(self):
        return self._run_as_non_root

    @run_as_non_root.setter
    def run_as_non_root(self, r=None):
        if not isinstance(r, bool):
            raise SyntaxError('SecurityContext: run_as_non_root: [ {} ] is invalid.'.format(r))
        self._run_as_non_root = r

    # ------------------------------------------------------------------------------------- runAsUser

    # May also be set in PodSecurityContext.
    # If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.

    @property
    def run_as_user(self):
        return self._run_as_user

    @run_as_user.setter
    def run_as_user(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('SecurityContext: run_as_user: [ {} ] is invalid.'.format(r))
        self._run_as_user = r

    # ------------------------------------------------------------------------------------- seLinuxOptions

    # May also be set in PodSecurityContext.
    # If set in both SecurityContext and PodSecurityContext, the value specified in SecurityContext takes precedence.

    @property
    def se_linux_options(self):
        return self._se_linux_options

    @se_linux_options.setter
    def se_linux_options(self, o=None):
        if not isinstance(o, SELinuxOptions):
            raise SyntaxError('SecurityContext: se_linux_options: [ {} ] is invalid.'.format(o))
        self._se_linux_options = o

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.capabilities is not None:
            data['capabilities'] = self.capabilities.serialize()
        if self.privileged is not None:
            data['privileged'] = self.privileged
        if self.read_only_root_filesystem is not None:
            data['readOnlyRootFilesystem'] = self.read_only_root_filesystem
        if self.run_as_non_root is not None:
            data['runAsNonRoot'] = self.run_as_non_root
        if self.run_as_user is not None:
            data['runAsUser'] = self.run_as_user
        if self.se_linux_options is not None:
            data['seLinuxOptions'] = self.se_linux_options.serialize()
        return data




#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1beta1.RollingUpdateDeployment import RollingUpdateDeployment
from kubernetes.utils import is_valid_string


class DeploymentStrategy(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_deploymentstrategy
    """

    def __init__(self, model=None):
        super(DeploymentStrategy, self).__init__()

        self._type = None
        self._rolling_update = RollingUpdateDeployment()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'type' in model:
            self.type = model['type']
        if 'rollingUpdate' in model:
            self.rolling_update = RollingUpdateDeployment(model['rollingUpdate'])

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('DeploymentStrategy: type: [ {} ] is invalid.'.format(t))
        self._type = t

    # ------------------------------------------------------------------------------------- rollingUpdate

    @property
    def rolling_update(self):
        return self._rolling_update

    @rolling_update.setter
    def rolling_update(self, ru=None):
        if not isinstance(ru, RollingUpdateDeployment):
            raise SyntaxError('DeploymentStrategy: rolling_update: [ {} ] is invalid.'.format(ru))
        self._rolling_update = ru

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.type is not None:
            data['type'] = self.type
        if self.rolling_update is not None:
            data['rollingUpdate'] = self.rolling_update.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class NamespaceStatus(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_namespacestatus
    """

    def __init__(self, model=None):
        super(NamespaceStatus, self).__init__()

        self._phase = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'phase' in model:
            self.finalizers = model['phase']

    # --------------------------------------------------------------------------------- finalizers

    @property
    def phase(self):
        return self._phase

    @phase.setter
    def phase(self, p=None):
        if not isinstance(p, str):
            raise SyntaxError('NamespaceStatus: phase: [ {} ] is invalid. Must be a string.'.format(p))
        self._phase = p

    # --------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.phase is not None:
            data['phase'] = self.phase
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.Container import Container
from kubernetes.models.v1.PodSecurityContext import PodSecurityContext
from kubernetes.models.v1.Volume import Volume
from kubernetes.models.v1.Affinity import Affinity
from kubernetes.models.v1.Toleration import Toleration
from kubernetes.utils import is_valid_list, filter_model, is_valid_string


class PodSpec(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_podspec
    """

    VALID_DNS_POLICIES = ['ClusterFirst', 'Default']
    VALID_RESTART_POLICIES = ['Always', 'OnFailure', 'Never']

    def __init__(self, model=None):
        super(PodSpec, self).__init__()

        self._active_deadline_seconds = None
        self._affinity = None
        self._containers = []
        self._automount_service_account_token = True
        self._dns_policy = 'Default'
        self._host_ipc = None
        self._host_network = None
        self._host_pid = None
        self._hostname = None
        self._image_pull_secrets = None
        self._node_name = None
        self._node_selector = {}
        self._restart_policy = 'Always'
        self._security_context = PodSecurityContext()
        self._service_account = None  # deprecated
        self._service_account_name = None
        self._subdomain = None
        self._termination_grace_period_seconds = 30
        self._tolerations = []
        self._volumes = []

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'activeDeadlineSeconds' in model:
            self.active_deadline_seconds = model['activeDeadlineSeconds']
        if 'affinity' in model:
            self.affinity = Affinity(model['affinity'])
        if 'containers' in model:
            containers = []
            for c in model['containers']:
                container = Container(c)
                containers.append(container)
            self.containers = containers
        if 'dnsPolicy' in model:
            self.dns_policy = model['dnsPolicy']
        if 'hostIPC' in model:
            self.host_ipc = model['hostIPC']
        if 'hostNetwork' in model:
            self.host_network = model['hostNetwork']
        if 'hostPID' in model:
            self.host_pid = model['hostPID']
        if 'hostname' in model:
            self.hostname = model['hostname']
        if 'imagePullSecrets' in model:
            self.image_pull_secrets = model['imagePullSecrets']
        if 'nodeName' in model:
            self.node_name = model['nodeName']
        if 'nodeSelector' in model:
            self.node_selector = model['nodeSelector']
        if 'restartPolicy' in model:
            self.restart_policy = model['restartPolicy']
        if 'securityContext' in model:
            self.security_context = PodSecurityContext(model['securityContext'])
        if 'serviceAccount' in model:
            self.service_account = model['serviceAccount']
        if 'serviceAccountName' in model:
            self.service_account_name = model['serviceAccountName']
        if 'subdomain' in model:
            self.subdomain = model['subdomain']
        if 'terminationGracePeriodSeconds' in model:
            self.termination_grace_period_seconds = model['terminationGracePeriodSeconds']
        if 'tolerations' in model:
            tolerations = []
            for t in model['tolerations']:
                tol = Toleration(t)
                tolerations.append(tol)
            self.tolerations = tolerations
        if 'volumes' in model:
            volumes = []
            for v in model['volumes']:
                volume = Volume(v)
                volumes.append(volume)
            self.volumes = volumes

    # ------------------------------------------------------------------------------------- add

    def add_container(self, container=None):
        if not isinstance(container, Container):
            raise SyntaxError('PodSpec.add_container90: container: [ {0} ] is invalid.'.format(container.__class__.__name__))
        self._containers.append(container)
        return self

    def add_volume(self, volume=None):
        if not isinstance(volume, Volume):
            raise SyntaxError('PodSpec.add_volume(): volume: [ {0} ] is invalid'.format(volume))
        self._volumes.append(volume)

    def add_image_pull_secrets(self, secrets=None):
        if not is_valid_list(secrets, dict):
            raise SyntaxError('PodSpec.add_image_pull_secrets() secrets : [ {0} ] is invalid.'.format(secrets))

        s = self.image_pull_secrets
        if s is None:
            l = secrets
        else:
            l = s + secrets

        self.image_pull_secrets = [dict(t) for t in set([tuple(d.items()) for d in l])]
        return self

    # ------------------------------------------------------------------------------------- del

    def del_node_name(self):
        self._node_name = None

    # ------------------------------------------------------------------------------------- active deadline seconds

    @property
    def active_deadline_seconds(self):
        return self._active_deadline_seconds

    @active_deadline_seconds.setter
    def active_deadline_seconds(self, secs=None):
        if not isinstance(secs, int):
            raise SyntaxError('PodSpec: active_deadline_seconds: [ {0} ] is invalid.'.format(secs))
        self._active_deadline_seconds = secs

    # ------------------------------------------------------------------------------------- affinity

    @property
    def affinity(self):
        return self._affinity

    @affinity.setter
    def affinity(self, a=None):
        if not isinstance(a, Affinity):
            raise SyntaxError('PodSpec: affinity: [ {} ] is invalid.'.format(a))
        self._affinity = a

    # ------------------------------------------------------------------------------------- automount service account token

    @property
    def automount_service_account_token(self):
        return self._automount_service_account_token

    @automount_service_account_token.setter
    def automount_service_account_token(self, automount=None):
        if not isinstance(automount, bool):
            raise SyntaxError('PodSpec: automount_service_account_token: [ {0} ] is invalid.'.format(automount))
        self._automount_service_account_token = automount

    # ------------------------------------------------------------------------------------- containers

    @property
    def containers(self):
        return self._containers

    @containers.setter
    def containers(self, containers=None):
        if not is_valid_list(containers, Container):
            raise SyntaxError('PodSpec: containers: [ {0} ] is invalid.'.format(containers))
        self._containers = containers

    def set_container_image(self, name=None, image=None):
        if not is_valid_string(name):
            raise SyntaxError('PodSpec: name: [ {0} ] is invalid.')
        if not is_valid_string(image):
            raise SyntaxError('PodSpec: image: [ {0} ] is invalid.')
        for c in self.containers:
            if c.name == name:
                c.image(image=image)
                break
        return self

    # ------------------------------------------------------------------------------------- dnsPolicy

    @property
    def dns_policy(self):
        return self._dns_policy

    @dns_policy.setter
    def dns_policy(self, dns_policy=None):
        if dns_policy not in PodSpec.VALID_DNS_POLICIES:
            raise SyntaxError('PodSpec: dns_policy: [ {0} ] is invalid.'.format(dns_policy))
        self._dns_policy = dns_policy

    # ------------------------------------------------------------------------------------- hostIPC

    @property
    def host_ipc(self):
        return self._host_ipc

    @host_ipc.setter
    def host_ipc(self, ipc=None):
        if not isinstance(ipc, bool):
            raise SyntaxError('PodSpec: host_ipc: [ {0} ] is invalid.'.format(ipc))
        self._host_ipc = ipc

    # ------------------------------------------------------------------------------------- hostPID

    @property
    def host_pid(self):
        return self._host_pid

    @host_pid.setter
    def host_pid(self, pid=None):
        if not isinstance(pid, bool):
            raise SyntaxError('PodSpec: host_pid: [ {0} ] is invalid.'.format(pid))
        self._host_pid = pid

    # ------------------------------------------------------------------------------------- hostNetwork

    @property
    def host_network(self):
        return self._host_network

    @host_network.setter
    def host_network(self, hn=None):
        if not isinstance(hn, bool):
            raise SyntaxError('PodSpec: host_network: [ {0} ] is invalid.'.format(hn))
        self._host_network = hn

    # ------------------------------------------------------------------------------------- hostname

    @property
    def hostname(self):
        return self._hostname

    @hostname.setter
    def hostname(self, hn=None):
        if not is_valid_string(hn):
            raise SyntaxError('PodSpec: hostname: [ {0} ] is invalid.'.format(hn))
        self._hostname = hn

    # ------------------------------------------------------------------------------------- image pull secrets

    @property
    def image_pull_secrets(self):
        return self._image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, secrets=None):
        if not is_valid_list(secrets, dict):
            raise SyntaxError('PodSpec: image_pull_secrets: [ {0} ] is invalid.'.format(secrets))
        self._image_pull_secrets = secrets

    # ------------------------------------------------------------------------------------- node selector

    @property
    def node_selector(self):
        return self._node_selector

    @node_selector.setter
    def node_selector(self, selector=None):
        if not isinstance(selector, dict):
            raise SyntaxError('PodSpec: node_selector: [ {0} ] is invalid.'.format(selector))
        self._node_selector = selector

    # ------------------------------------------------------------------------------------- node name

    @property
    def node_name(self):
        return self._node_name

    @node_name.setter
    def node_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('PodSpec: node_name: [ {0} ] is invalid.'.format(name))
        self._node_name = name

    # ------------------------------------------------------------------------------------- restart policy

    @property
    def restart_policy(self):
        return self._restart_policy

    @restart_policy.setter
    def restart_policy(self, policy=None):
        if policy not in PodSpec.VALID_RESTART_POLICIES:
            raise SyntaxError('PodSpec: policy: [ {0} ] is invalid.'.format(policy))
        self._restart_policy = policy

    # ------------------------------------------------------------------------------------- security context

    @property
    def security_context(self):
        return self._security_context

    @security_context.setter
    def security_context(self, context=None):
        if not isinstance(context, PodSecurityContext):
            raise SyntaxError('PodSpec: pod_security_context: [ {0} ] is invalid.'.format(context))
        self._security_context = context

    # ------------------------------------------------------------------------------------- service account name

    # TODO(froch): remove 'service_account' since it's deprecated.
    # We're leaving this in until 'serviceAccount' is rejected by the Kubernetes API server.
    # You should be using 'serviceAccountName'. We're forcing use of it under the hood.

    @property
    def service_account(self):
        return self._service_account_name

    @service_account.setter
    def service_account(self, san=None):
        if not is_valid_string(san):
            raise SyntaxError('PodSpec: service_account: [ {0} ] is invalid.'.format(san))
        self._service_account_name = san
        self._service_account = san

    @property
    def service_account_name(self):
        return self._service_account_name

    @service_account_name.setter
    def service_account_name(self, san=None):
        if not is_valid_string(san):
            raise SyntaxError('PodSpec: service_account_name: [ {0} ] is invalid.'.format(san))
        self._service_account_name = san
        self._service_account = san

    # ------------------------------------------------------------------------------------- subdomain

    @property
    def subdomain(self):
        return self._subdomain

    @subdomain.setter
    def subdomain(self, subdomain=None):
        if not is_valid_string(subdomain):
            raise SyntaxError('PodSpec: subdomain: [ {0} ] is invalid.'.format(subdomain))
        self._subdomain = subdomain

    # ------------------------------------------------------------------------------------- termination grace period

    @property
    def termination_grace_period_seconds(self):
        return self._termination_grace_period_seconds

    @termination_grace_period_seconds.setter
    def termination_grace_period_seconds(self, secs=None):
        if isinstance(secs, str) and secs.isdigit():
            secs = int(secs)
        if not isinstance(secs, int) or not secs >= 0:
            raise SyntaxError('PodSpec: termination_grace_period_seconds: [ {0} ] is invalid.'.format(secs))
        self._termination_grace_period_seconds = secs

    # ------------------------------------------------------------------------------------- tolerations

    @property
    def tolerations(self):
        return self._tolerations

    @tolerations.setter
    def tolerations(self, t=None):
        if not is_valid_list(t, Toleration):
            raise SyntaxError('PodSpec: tolerations: [ {} ] is invalid.'.format(t))
        self._tolerations = t

    # ------------------------------------------------------------------------------------- volumes

    @property
    def volumes(self):
        return self._volumes

    @volumes.setter
    def volumes(self, volumes=None):
        if not is_valid_list(volumes, Volume):
            raise SyntaxError('PodSpec: volumes: [ {0} ] is invalid.'.format(volumes))
        self._volumes = volumes

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.active_deadline_seconds:
            data['activeDeadlineSeconds'] = self.active_deadline_seconds
        if self.affinity:
            data['affinity'] = self.affinity.serialize()
        if not self.automount_service_account_token:
            data['automountServiceAccountToken'] = self.automount_service_account_token
        if self.containers:
            data['containers'] = []
            for c in self.containers:
                data['containers'].append(c.serialize())
        if self.dns_policy:
            data['dnsPolicy'] = self.dns_policy
        if self.host_ipc:
            data['hostIPC'] = self.host_ipc
        if self.host_network is not None:
            data['hostNetwork'] = self.host_network
        if self.host_pid is not None:
            data['hostPID'] = self.host_pid
        if self.hostname:
            data['hostname'] = self.hostname
        if self.host_ipc is not None:
            data['hostIPC'] = self.host_ipc
        if self.image_pull_secrets:
            data['imagePullSecrets'] = self.image_pull_secrets
        if self.node_name:
            data['nodeName'] = self.node_name
        if self.node_selector:
            data['nodeSelector'] = self.node_selector
        if self.restart_policy:
            data['restartPolicy'] = self.restart_policy
        if self.service_account:
            data['serviceAccount'] = self.service_account
        if self.service_account_name:
            data['serviceAccountName'] = self.service_account_name
        if self.subdomain:
            data['subdomain'] = self.subdomain
        if self.termination_grace_period_seconds:
            data['terminationGracePeriodSeconds'] = self.termination_grace_period_seconds
        if self.tolerations:
            tolerations = []
            for t in self.tolerations:
                tol = t.serialize()
                tolerations.append(tol)
            data['tolerations'] = tolerations
        if self.volumes:
            data['volumes'] = []
            for v in self.volumes:
                data['volumes'].append(v.serialize())
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ContainerStateRunning import ContainerStateRunning
from kubernetes.models.v1.ContainerStateTerminated import ContainerStateTerminated
from kubernetes.models.v1.ContainerStateWaiting import ContainerStateWaiting
from kubernetes.utils import filter_model


class ContainerState(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerstate
    """

    def __init__(self, model=None):
        super(ContainerState, self).__init__()

        self._waiting = None
        self._running = None
        self._terminated = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'waiting' in model:
            self.waiting = ContainerStateWaiting(model['waiting'])
        if 'running' in model:
            self.running = ContainerStateRunning(model['running'])
        if 'terminated' in model:
            self.terminated = ContainerStateTerminated(model['terminated'])

    # ------------------------------------------------------------------------------------- waiting

    @property
    def waiting(self):
        return self._waiting

    @waiting.setter
    def waiting(self, state=None):
        if not isinstance(state, ContainerStateWaiting):
            raise SyntaxError('ContainerState: waiting: [ {0} ] is invalid.'.format(state))
        self._waiting = state

    # ------------------------------------------------------------------------------------- running

    @property
    def running(self):
        return self._running

    @running.setter
    def running(self, state=None):
        if not isinstance(state, ContainerStateRunning):
            raise SyntaxError('ContainerState: running: [ {0} ] is invalid.'.format(state))
        self._running = state

    # ------------------------------------------------------------------------------------- terminated

    @property
    def terminated(self):
        return self._terminated

    @terminated.setter
    def terminated(self, state=None):
        if not isinstance(state, ContainerStateTerminated):
            raise SyntaxError('ContainerState: terminated: [ {0} ] is invalid.'.format(state))
        self._terminated = state

    # ------------------------------------------------------------------------------------- serialized

    def serialize(self):
        data = {}
        if self.waiting is not None:
            data['waiting'] = self.waiting.serialize()
        if self.running is not None:
            data['running'] = self.running.serialize()
        if self.terminated is not None:
            data['terminated'] = self.terminated.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json
import uuid

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.Secret import Secret


class K8sSecret(K8sObject):

    def __init__(self, config=None, name=None):
        super(K8sSecret, self).__init__(
            config=config,
            obj_type='Secret',
            name=name
        )

    # -------------------------------------------------------------------------------------  override

    def get(self):
        self.model = Secret(self.get_model())
        return self

    def create(self):
        super(K8sSecret, self).create()
        self.get()
        return self

    def update(self):
        super(K8sSecret, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sSecret, self).list()
        secrets = list(map(lambda x: Secret(x), ls))
        if pattern is not None:
            secrets = list(filter(lambda x: pattern in x.name, secrets))
        k8s = []
        for x in secrets:
            j = K8sSecret(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # -------------------------------------------------------------------------------------  image pull secrets

    @staticmethod
    def create_image_pull_secret(config=None, prefix=None, name=None, data=None):
        s = Secret()

        if name is not None:
            s.name = "{0}-docker-{1}".format(name, str(uuid.uuid4().hex[:5]))
        elif name is None and prefix is not None:
            s.name = "{0}-docker-{1}".format(prefix, str(uuid.uuid4().hex[:5]))
        else:
            s.name = "docker-{0}".format(str(uuid.uuid4().hex[:5]))

        s.dockerconfigjson = data
        k8s = K8sSecret(config=config, name=s.name)
        k8s.model = s
        k8s.create()
        return k8s

    @staticmethod
    def list_image_pull_secrets(config=None):
        _list = K8sSecret(config=config, name="yo").list()
        secrets = []
        for x in _list:
            if x.type == Secret.K8s_TYPE_DOCKER_CONFIG:
                secrets.append(x)
        return secrets

    @staticmethod
    def image_pull_secret_with_name(config=None, name=None):
        _secrets = K8sSecret.list_image_pull_secrets(config=config)
        _list = list(filter(lambda x: x.name == name, _secrets))
        if len(_list):
            return _list[0]
        return None

    # -------------------------------------------------------------------------------------  service accounts

    @staticmethod
    def create_service_account_api_token(config=None, name=None):
        s = Secret()
        s.name = "{}-token-{}".format(name, str(uuid.uuid4().hex[:5]))
        s.add_annotation(Secret.K8s_ANNOTATION_SERVICE_ACCOUNT_NAME, name)
        s.type = Secret.K8s_TYPE_SERVICE_ACCOUNT
        k8s = K8sSecret(config=config, name=s.name)
        k8s.model = s
        k8s.create()
        return k8s

    @staticmethod
    def api_tokens_for_service_account(config=None, name=None):
        _list = K8sSecret(config=config, name="yo").list()
        _tokens = []
        for x in _list:
            if x.type == Secret.K8s_TYPE_SERVICE_ACCOUNT:
                if x.annotations[Secret.K8s_ANNOTATION_SERVICE_ACCOUNT_NAME] == name:
                    _tokens.append(x)
        return _tokens

    # ------------------------------------------------------------------------------------- data

    @property
    def data(self):
        return self.model.data

    @data.setter
    def data(self, data=None):
        self.model.data = data

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self.model.type

    @type.setter
    def type(self, t=None):
        self.model.type = t

    # ------------------------------------------------------------------------------------- dockerconfigjson

    @property
    def dockerconfigjson(self):
        data = None
        if self.model.dockerconfigjson is not None:
            data = json.loads(self.model.dockerconfigjson)
        return data

    @dockerconfigjson.setter
    def dockerconfigjson(self, secret=None):
        self.model.dockerconfigjson = secret

    # ------------------------------------------------------------------------------------- set

    def set_service_account_token(self, account_name=None, account_uid=None, token=None,
                                  kubecfg_data=None, cacert=None):
        self.model.set_service_account_token(
            account_name=account_name,
            account_uid=account_uid,
            token=token,
            kubecfg_data=kubecfg_data,
            cacert=cacert
        )
        return self



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class ResourceFieldSelector(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_resourcefieldselector
    """

    def __init__(self, model=None):
        super(ResourceFieldSelector, self).__init__()

        self._container_name = None
        self._resource = None
        self._divisor = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'containerName' in model:
            self.container_name = model['containerName']
        if 'resource' in model:
            self.resource = model['resource']
        if 'divisor' in model:
            self.divisor = model['divisor']

    # ------------------------------------------------------------------------------------- containerName

    @property
    def container_name(self):
        return self._container_name

    @container_name.setter
    def container_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ResourceFieldSelector: container_name: [ {} ] is not None'.format(name))
        self._container_name = name

    # ------------------------------------------------------------------------------------- resource

    @property
    def resource(self):
        return self._resource

    @resource.setter
    def resource(self, res=None):
        if not is_valid_string(res):
            raise SyntaxError('ResourceFieldSelector: resource: [ {} ] is not None'.format(res))
        self._resource = res

    # ------------------------------------------------------------------------------------- divisor

    @property
    def divisor(self):
        return self._divisor

    @divisor.setter
    def divisor(self, div=None):
        if not is_valid_string(div):
            raise SyntaxError('ResourceFieldSelector: divisor: [ {} ] is not None'.format(div))
        self._divisor = div

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.container_name is not None:
            data['containerName'] = self.container_name
        if self.resource is not None:
            data['resource'] = self.resource
        if self.divisor is not None:
            data['divisor'] = self.divisor
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class ContainerStateRunning(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerstaterunning
    """

    def __init__(self, model=None):
        super(ContainerStateRunning, self).__init__()

        self._started_at = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'startedAt' in model:
            self.started_at = model['startedAt']

    # ------------------------------------------------------------------------------------- reason

    @property
    def started_at(self):
        return self._started_at

    @started_at.setter
    def started_at(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('CoùntainerStateRunning: started_at: [ {0} ] is invalid.'.format(time))
        self._started_at = time

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.started_at is not None:
            data['startedAt'] = self.started_at
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PersistentVolumeSpec import PersistentVolumeSpec
from kubernetes.utils import is_valid_string, is_valid_list, is_valid_dict


class PersistentVolumeClaimStatus(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_persistentvolumeclaimstatus
    """

    def __init__(self, model=None):
        super(PersistentVolumeClaimStatus, self).__init__()

        self._phase = None
        self._access_modes = None
        self._capacity = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'phase' in model:
            self.phase = model['phase']
        if 'accessModes' in model:
            self.access_modes = model['accessModes']
        if 'capacity' in model:
            self.capacity = model['capacity']

    # ------------------------------------------------------------------------------------- phase

    @property
    def phase(self):
        return self._phase

    @phase.setter
    def phase(self, p=None):
        if not is_valid_string(p):
            raise SyntaxError('PersistentVolumeClaimStatus: phase: [ {} ] is invalid.'.format(p))
        self._phase = p

    # ------------------------------------------------------------------------------------- accessModes

    @property
    def access_modes(self):
        return self._access_modes

    @access_modes.setter
    def access_modes(self, modes=None):
        if not is_valid_list(modes, str):
            raise SyntaxError('PersistentVolumeClaimStatus: access_modes: [ {} ] is invalid.'.format(modes))
        filtered = list(filter(lambda x: x in PersistentVolumeSpec.VALID_ACCESS_MODES, modes))
        self._access_modes = filtered

    # ------------------------------------------------------------------------------------- capacity

    @property
    def capacity(self):
        return self._capacity

    @capacity.setter
    def capacity(self, c=None):
        if not is_valid_dict(c):
            raise SyntaxError('PersistentVolumeClaimStatus: capacity: [ {} ] is invalid.'.format(c))
        self._capacity = c

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.phase is not None:
            data['phase'] = self.phase
        if self.access_modes is not None:
            data['accessModes'] = self.access_modes
        if self.capacity is not None:
            data['capacity'] = self.capacity
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class ObjectReference(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_objectreference
    """

    def __init__(self, model=None):
        super(ObjectReference, self).__init__()

        self._kind = None
        self._namespace = None
        self._name = None
        self._uid = None
        self._api_version = None
        self._resource_version = None
        self._field_path = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'kind' in model:
            self.kind = model['kind']
        if 'namespace' in model:
            self.namespace = model['namespace']
        if 'name' in model:
            self.name = model['name']
        if 'uid' in model:
            self.uid = model['uid']
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'resourceVersion' in model:
            self.resource_version = model['resourceVersion']
        if 'fieldPath' in model:
            self.field_path = model['fieldPath']

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('ObjectReference: kind: [ {} ] is invalid.'.format(k))
        self._kind = k

    # ------------------------------------------------------------------------------------- namespace

    @property
    def namespace(self):
        return self._namespace

    @namespace.setter
    def namespace(self, ns=None):
        if not is_valid_string(ns):
            raise SyntaxError('ObjectReference: namespace: [ {} ] is invalid.'.format(ns))
        self._namespace = ns

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ObjectReference: name: [ {} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- uid

    @property
    def uid(self):
        return self._uid

    @uid.setter
    def uid(self, uid=None):
        if not is_valid_string(uid):
            raise SyntaxError('ObjectReference: uid: [ {} ] is invalid.'.format(uid))
        self._uid = uid

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('ObjectReference: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- resourceVersion

    @property
    def resource_version(self):
        return self._resource_version

    @resource_version.setter
    def resource_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('ObjectReference: resource_version: [ {} ] is invalid.'.format(v))
        self._resource_version = v

    # ------------------------------------------------------------------------------------- fieldPath

    @property
    def field_path(self):
        return self._field_path

    @field_path.setter
    def field_path(self, path=None):
        if not is_valid_string(path):
            raise SyntaxError('ObjectReference: field_path: [ {} ] is invalid.'.format(path))
        self._field_path = path

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.kind is not None:
            data['kind'] = self.kind
        if self.namespace is not None:
            data['namespace'] = self.namespace
        if self.name is not None:
            data['name'] = self.name
        if self.uid is not None:
            data['uid'] = self.uid
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.resource_version is not None:
            data['resourceVersion'] = self.resource_version
        if self.field_path is not None:
            data['fieldPath'] = self.field_path
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.NodeAffinity import NodeAffinity
from kubernetes.models.v1.PodAffinity import PodAffinity
from kubernetes.models.v1.PodAntiAffinity import PodAntiAffinity


class Affinity(object):
    """
    https://kubernetes.io/docs/concepts/configuration/assign-pod-node/#affinity-and-anti-affinity
    """

    def __init__(self, model=None):
        super(Affinity, self).__init__()

        self._node_affinity = None
        self._pod_affinity = None
        self._pod_anti_affinity = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'nodeAffinity' in model:
            self.node_affinity = NodeAffinity(model['nodeAffinity'])
        if 'podAffinity' in model:
            self.pod_affinity = PodAffinity(model['podAffinity'])
        if 'podAntiAffinity' in model:
            self.pod_anti_affinity = PodAntiAffinity(model['podAntiAffinity'])

    # ------------------------------------------------------------------------------------- nodeAffinity

    @property
    def node_affinity(self):
        return self._node_affinity

    @node_affinity.setter
    def node_affinity(self, na=None):
        if not isinstance(na, NodeAffinity):
            raise SyntaxError('Affinity: node_affinity: [ {} ] is invalid.'.format(na))
        self._node_affinity = na

    # ------------------------------------------------------------------------------------- podAffinity

    @property
    def pod_affinity(self):
        return self._pod_affinity

    @pod_affinity.setter
    def pod_affinity(self, pa=None):
        if not isinstance(pa, PodAffinity):
            raise SyntaxError('Affinity: pod_affinity: [ {} ] is invalid.'.format(pa))
        self._pod_affinity = pa

    # ------------------------------------------------------------------------------------- podAntiAffinity

    @property
    def pod_anti_affinity(self):
        return self._pod_anti_affinity

    @pod_anti_affinity.setter
    def pod_anti_affinity(self, paa=None):
        if not isinstance(paa, PodAntiAffinity):
            raise SyntaxError('Affinity: pod_anti_affinity: [ {} ] is invalid.'.format(paa))
        self._pod_anti_affinity = paa

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.node_affinity is not None:
            data['nodeAffinity'] = self.node_affinity.serialize()
        if self.pod_affinity is not None:
            data['podAffinity'] = self.pod_affinity.serialize()
        if self.pod_anti_affinity is not None:
            data['podAntiAffinity'] = self.pod_anti_affinity.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class VolumeMount(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_volumemount
    """

    def __init__(self, model=None):
        super(VolumeMount, self).__init__()

        self._name = None
        self._mount_path = None
        self._read_only = None
        self._sub_path = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model):
        if 'mountPath' in model:
            self.mount_path = model['mountPath']
        if 'name' in model:
            self.name = model['name']
        if 'readOnly' in model:
            self.read_only = model['readOnly']
        if 'subPath' in model:
            self.sub_path = model['subPath']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('VolumeMount: name: [ {0} ] is invalid'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- mount_path

    @property
    def mount_path(self):
        return self._mount_path

    @mount_path.setter
    def mount_path(self, path=None):
        if not is_valid_string(path):
            raise SyntaxError('VolumeMount: mount_path: [ {0} ] is invalid.'.format(path))
        self._mount_path = path

    # ------------------------------------------------------------------------------------- read_only

    @property
    def read_only(self):
        return self._read_only

    @read_only.setter
    def read_only(self, ro=False):
        if not isinstance(ro, bool):
            raise SyntaxError('VolumeMount: read_only: [ {0} ] is invalid.'.format(ro))
        self._read_only = ro

    # ------------------------------------------------------------------------------------- sub_path

    @property
    def sub_path(self):
        return self._sub_path

    @sub_path.setter
    def sub_path(self, path=None):
        if path is None:
            path = ""  # default; volume's root
        if not is_valid_string(path):
            raise SyntaxError('VolumeMount: sub_path: [ {0} ] is invalid.'.format(path))
        self._sub_path = path

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name:
            data['name'] = self.name
        if self.read_only:
            data['readOnly'] = self.read_only
        if self.mount_path:
            data['mountPath'] = self.mount_path
        if self.sub_path:
            data['subPath'] = self.sub_path
        return data



import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# See https://github.com/pybuilder/pybuilder/issues/56
del os.link

# install_reqs = parse_requirements(os.path.abspath(__file__).replace('setup.py', 'requirements.txt'),
#                                   session=PipSession())
# reqs = [str(ir.req) for ir in install_reqs]


def version():
    with open(os.path.abspath(__file__).replace('setup.py', 'version.meta'), 'r') as v:
        return v.read()


setup(
    name='kubernetes-py',
    version=version(),
    description='A python module for Kubernetes.',
    author='mnubo inc.',
    author_email='it-admin@mnubo.com',
    url='https://github.com/mnubo/kubernetes-py',
    download_url='https://github.com/mnubo/kubernetes-py/tarball/' + version(),
    keywords=['kubernetes', 'k8s'],
    packages=[
        'kubernetes',
        'kubernetes.models',
        'kubernetes.models.unversioned',
        'kubernetes.models.v1',
        'kubernetes.models.v1alpha1',
        'kubernetes.models.v1beta1',
        'kubernetes.models.v2alpha1',
        'kubernetes.utils'
    ],
    install_requires=[
        'six>=1.10.0',
        'PyYAML==3.11',
        'requests>=2.10.0',
        'uuid==1.30',
        'python-dateutil>=2.6.0'
    ],
    scripts=[],
    test_suite='nose.collector',
    tests_require=[
        'nose',
        'nose-cover3'
    ],
    include_package_data=True,
    zip_safe=False
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.Service import Service


class K8sService(K8sObject):

    def __init__(self, config=None, name=None):
        super(K8sService, self).__init__(
            config=config,
            name=name,
            obj_type='Service'
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sService, self).create()
        self.get()
        return self

    def update(self):
        super(K8sService, self).update()
        self.get()
        return self

    def list(self, pattern=None):
        ls = super(K8sService, self).list()
        svcs = list(map(lambda x: Service(x), ls))
        if pattern is not None:
            svcs = list(filter(lambda x: pattern in x.name, svcs))
        k8s = []
        for x in svcs:
            j = K8sService(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- add

    def add_annotation(self, k=None, v=None):
        self.model.add_annotation(k=k, v=v)
        return self

    def add_label(self, k=None, v=None):
        self.model.add_label(k=k, v=v)
        return self

    def add_port(self, name=None, port=None, target_port=None, protocol=None, node_port=None):
        if isinstance(target_port, int):
            target_port = str(target_port)
        self.model.add_port(
            name=name,
            port=port,
            target_port=target_port,
            protocol=protocol,
            node_port=node_port
        )
        return self

    def add_selector(self, selector=None):
        self.model.add_selector(selector=selector)
        return self

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = Service(self.get_model())
        return self

    def get_annotation(self, k=None):
        if k in self.model.metadata.annotations:
            return self.model.metadata.annotations[k]
        return None

    def get_label(self, k=None):
        if k in self.model.metadata.labels:
            return self.model.metadata.labels[k]
        return None

    # ------------------------------------------------------------------------------------- clusterIP

    @property
    def cluster_ip(self):
        return self.model.spec.cluster_ip

    @cluster_ip.setter
    def cluster_ip(self, ip=None):
        self.model.spec.cluster_ip = ip

    # ------------------------------------------------------------------------------------- externalIPs

    @property
    def external_ips(self):
        return self.model.spec.external_ips

    @external_ips.setter
    def external_ips(self, ips=None):
        self.model.spec.external_ips = ips

    # ------------------------------------------------------------------------------------- loadBalancerIP

    @property
    def load_balancer_ip(self):
        return self.model.spec.load_balancer_ip

    @load_balancer_ip.setter
    def load_balancer_ip(self, ip=None):
        self.model.spec.load_balancer_ip = ip

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.name

    @name.setter
    def name(self, name=None):
        self.model.name = name

    # ------------------------------------------------------------------------------------- namespace

    @property
    def namespace(self):
        return self.model.metadata.namespace

    @namespace.setter
    def namespace(self, nspace=None):
        self.model.metadata.namespace = nspace

    # ------------------------------------------------------------------------------------- ports

    @property
    def ports(self):
        return self.model.spec.ports

    @ports.setter
    def ports(self, ports=None):
        self.model.spec.ports = ports

    # ------------------------------------------------------------------------------------- sessionAffinity

    @property
    def session_affinity(self):
        return self.model.spec.session_affinity

    @session_affinity.setter
    def session_affinity(self, sa=None):
        self.model.spec.session_affinity = sa

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self.model.spec.selector

    @selector.setter
    def selector(self, s=None):
        self.model.spec.selector = s

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self.model.spec.type

    @type.setter
    def type(self, t=None):
        self.model.spec.type = t

    # ------------------------------------------------------------------------------------- filter

    @staticmethod
    def get_by_name(config=None, name=None):
        service_list = []
        data = {'labelSelector': 'name={}'.format(name)}
        services = K8sService(config=config, name=name).get_with_params(data=data)
        for svc in services:
            service_name = Service(svc).metadata.name
            service_list.append(K8sService(config=config, name=service_name).get())
        return service_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PodTemplateSpec import PodTemplateSpec
from kubernetes.models.v1beta1.LabelSelector import LabelSelector


class ReplicaSetSpec(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_replicasetspec
    """

    def __init__(self, model=None):
        super(ReplicaSetSpec, self).__init__()

        self._replicas = 0
        self._selector = LabelSelector()
        self._template = PodTemplateSpec()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'replicas' in model:
            self.replicas = model['replicas']
        if 'selector' in model:
            self.selector = LabelSelector(model['selector'])
        if 'template' in model:
            self.template = PodTemplateSpec(model['template'])

    # ------------------------------------------------------------------------------------- replicas

    @property
    def replicas(self):
        return self._replicas

    @replicas.setter
    def replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('ReplicaSetSpec: replicas: [ {} ] is invalid.'.format(reps))
        self._replicas = reps

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self._selector

    @selector.setter
    def selector(self, sel=None):
        if not isinstance(sel, LabelSelector):
            raise SyntaxError('ReplicaSetSpec: selector: [ {} ] is invalid.'.format(sel))
        self._selector = sel

    # ------------------------------------------------------------------------------------- template

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, t=None):
        if not isinstance(t, PodTemplateSpec):
            raise SyntaxError('ReplicaSetSpec: template: [ {} ] is invalid.'.format(t))
        self._template = t

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.replicas is not None:
            data['replicas'] = self.replicas
        if self.selector is not None:
            data['selector'] = self.selector.serialize()
        if self.template is not None:
            data['template'] = self.template.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

# from kubernetes.models.v1.AttachedVolume import AttachedVolume
# from kubernetes.models.v1.ComponentCondition import ComponentCondition
# from kubernetes.models.v1.ComponentStatus import ComponentStatus
# from kubernetes.models.v1.ComponentStatusList import ComponentStatusList
# from kubernetes.models.v1.Container import Container
# from kubernetes.models.v1.ContainerImage import ContainerImage
# from kubernetes.models.v1.ContainerPort import ContainerPort
# from kubernetes.models.v1.ContainerStatus import ContainerStatus
# from kubernetes.models.v1.DaemonEndpoint import DaemonEndpoint
# from kubernetes.models.v1.DeleteOptions import DeleteOptions
# from kubernetes.models.v1.ExecAction import ExecAction
# from kubernetes.models.v1.HTTPGetAction import HTTPGetAction
# from kubernetes.models.v1.LoadBalancerIngress import LoadBalancerIngress
# from kubernetes.models.v1.LoadBalancerStatus import LoadBalancerStatus
# from kubernetes.models.v1.Namespace import Namespace
# from kubernetes.models.v1.NamespaceSpec import NamespaceSpec
# from kubernetes.models.v1.NamespaceStatus import NamespaceStatus
# from kubernetes.models.v1.Node import Node
# from kubernetes.models.v1.NodeAddress import NodeAddress
# from kubernetes.models.v1.NodeCondition import NodeCondition
# from kubernetes.models.v1.NodeDaemonEndpoints import NodeDaemonEndpoints
# from kubernetes.models.v1.NodeList import NodeList
# from kubernetes.models.v1.NodeStatus import NodeStatus
# from kubernetes.models.v1.NodeSpec import NodeSpec
# from kubernetes.models.v1.NodeSystemInfo import NodeSystemInfo
# from kubernetes.models.v1.ObjectMeta import ObjectMeta
# from kubernetes.models.v1.PersistentVolumeSpec import PersistentVolumeSpec
# from kubernetes.models.v1.Pod import Pod
# from kubernetes.models.v1.PodCondition import PodCondition
# from kubernetes.models.v1.PodSpec import PodSpec
# from kubernetes.models.v1.PodStatus import PodStatus
# from kubernetes.models.v1.PodTemplateSpec import PodTemplateSpec
# from kubernetes.models.v1.Probe import Probe
# from kubernetes.models.v1.ReplicationController import ReplicationController
# from kubernetes.models.v1.ReplicationControllerSpec import ReplicationControllerSpec
# from kubernetes.models.v1.ReplicationControllerStatus import ReplicationControllerStatus
# from kubernetes.models.v1.ResourceRequirements import ResourceRequirements
# from kubernetes.models.v1.Secret import Secret
# from kubernetes.models.v1.SecurityContext import SecurityContext
# from kubernetes.models.v1.Service import Service
# from kubernetes.models.v1.ServicePort import ServicePort
# from kubernetes.models.v1.ServiceSpec import ServiceSpec
# from kubernetes.models.v1.ServiceStatus import ServiceStatus
# from kubernetes.models.v1.TCPSocketAction import TCPSocketAction
# from kubernetes.models.v1.Volume import Volume
# from kubernetes.models.v1.VolumeMount import VolumeMount



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sExceptions import NotFoundException
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sPod import K8sPod
from kubernetes.models.v1beta1.ReplicaSet import ReplicaSet


class K8sReplicaSet(K8sObject):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_replicaset
    """

    REVISION_ANNOTATION = 'deployment.kubernetes.io/revision'
    REVISION_HISTORY_ANNOTATION = 'deployment.kubernetes.io/revision-history'

    def __init__(self, config=None, name=None):

        super(K8sReplicaSet, self).__init__(
            config=config,
            obj_type='ReplicaSet',
            name=name
        )

    # -------------------------------------------------------------------------------------  override

    def get(self):
        self.model = ReplicaSet(self.get_model())
        return self

    def list(self, pattern=None, reverse=True):
        ls = super(K8sReplicaSet, self).list()
        rsets = list(map(lambda x: ReplicaSet(x), ls))
        if pattern is not None:
            rsets = list(filter(lambda x: pattern in x.name, rsets))
        k8s = []
        for x in rsets:
            j = K8sReplicaSet(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        k8s.sort(key=lambda x: x.creation_timestamp, reverse=reverse)
        return k8s

    def delete(self, cascade=False):
        super(K8sReplicaSet, self).delete(cascade)
        if cascade:
            pods = K8sPod(config=self.config, name="yo").list(pattern=self.name)
            for pod in pods:
                try:
                    pod.delete(cascade)
                except NotFoundException:
                    pass
        return self

    # -------------------------------------------------------------------------------------  revision

    @property
    def revision(self):
        if self.REVISION_ANNOTATION in self.model.metadata.annotations:
            return self.model.metadata.annotations[self.REVISION_ANNOTATION]
        return None

    @revision.setter
    def revision(self, r=None):
        raise NotImplementedError(
            'K8sReplicaSet: revision is read-only.')

    # -------------------------------------------------------------------------------------  revision history

    @property
    def revision_history(self):
        if self.REVISION_HISTORY_ANNOTATION in self.model.metadata.annotations:
            comma_string = self.model.metadata.annotations[self.REVISION_HISTORY_ANNOTATION]
            version_array = comma_string.split(",")
            return map(lambda x: int(x), version_array)
        return None

    @revision_history.setter
    def revision_history(self, r=None):
        raise NotImplementedError(
            'K8sReplicaSet: revision_history is read-only.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_list


class Capabilities(object):
    """
    https://kubernetes.io/docs/api-reference/v1.5/#capabilities-v1-core
    """

    def __init__(self, model=None):
        super(Capabilities, self).__init__()

        self._add = None
        self._drop = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'add' in model:
            self.add = model['add']
        if 'drop' in model:
            self.drop = model['drop']

    # ------------------------------------------------------------------------------------- add

    @property
    def add(self):
        return self._add

    @add.setter
    def add(self, a=None):
        if not is_valid_list(a, str):
            raise SyntaxError('Capabilities: add: [ {} ] is invalid.'.format(a))
        self._add = a

    # ------------------------------------------------------------------------------------- drop

    @property
    def drop(self):
        return self._drop

    @drop.setter
    def drop(self, d=None):
        if not is_valid_list(d, str):
            raise SyntaxError('Capabilities: add: [ {} ] is invalid.'.format(d))
        self._drop = d

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.add is not None:
            data['add'] = self.add
        if self.drop is not None:
            data['drop'] = self.drop
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class DeploymentStatus(object):
    """
    http://kubernetes.io/docs/api-reference/extensions/v1beta1/definitions/#_v1beta1_deploymentstatus
    """

    def __init__(self, model=None):
        super(DeploymentStatus, self).__init__()

        self._observed_generation = None
        self._replicas = None
        self._updated_replicas = None
        self._available_replicas = None
        self._unavailable_replicas = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'observedGeneration' in model:
            self.observed_generation = model['observedGeneration']
        if 'replicas' in model:
            self.replicas = model['replicas']
        if 'updatedReplicas' in model:
            self.updated_replicas = model['updatedReplicas']
        if 'availableReplicas' in model:
            self.available_replicas = model['availableReplicas']
        if 'unavailableReplicas' in model:
            self.unavailable_replicas = model['unavailableReplicas']

    # ------------------------------------------------------------------------------------- observedGeneration

    @property
    def observed_generation(self):
        return self._observed_generation

    @observed_generation.setter
    def observed_generation(self, og=None):
        if not isinstance(og, int):
            raise SyntaxError('DeploymentStatus: observed_generation: [ {} ] is invalid.'.format(og))
        self._observed_generation = og

    # ------------------------------------------------------------------------------------- replicas

    @property
    def replicas(self):
        return self._replicas

    @replicas.setter
    def replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('DeploymentStatus: replicas: [ {} ] is invalid.'.format(reps))
        self._replicas = reps

    # ------------------------------------------------------------------------------------- updatedReplicas

    @property
    def updated_replicas(self):
        return self._updated_replicas

    @updated_replicas.setter
    def updated_replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('DeploymentStatus: updated_replicas: [ {} ] is invalid.'.format(reps))
        self._updated_replicas = reps

    # ------------------------------------------------------------------------------------- availableReplicas

    @property
    def available_replicas(self):
        return self._available_replicas

    @available_replicas.setter
    def available_replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('DeploymentStatus: available_replicas: [ {} ] is invalid.'.format(reps))
        self._available_replicas = reps

    # ------------------------------------------------------------------------------------- unavailableReplicas

    @property
    def unavailable_replicas(self):
        return self._unavailable_replicas

    @unavailable_replicas.setter
    def unavailable_replicas(self, reps=None):
        if not isinstance(reps, int):
            raise SyntaxError('DeploymentStatus: unavailable_replicas: [ {} ] is invalid.'.format(reps))
        self._unavailable_replicas = reps

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.observed_generation is not None:
            data['observedGeneration'] = self.observed_generation
        if self.replicas is not None:
            data['replicas'] = self.replicas
        if self.updated_replicas is not None:
            data['updatedReplicas'] = self.updated_replicas
        if self.available_replicas is not None:
            data['availableReplicas'] = self.available_replicas
        if self.unavailable_replicas is not None:
            data['unavailableReplicas'] = self.unavailable_replicas
        return data
